import { EventEmitter, Component, ChangeDetectionStrategy, forwardRef, ElementRef, NgZone, Renderer2, KeyValueDiffers, Input, Output, ViewChild, HostBinding, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { timer } from 'rxjs';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CodeMirrorComponent = /** @class */ (function () {
    function CodeMirrorComponent(elementRef, ngZone, renderer, _differs) {
        this.elementRef = elementRef;
        this.ngZone = ngZone;
        this.renderer = renderer;
        this._differs = _differs;
        this.value = null;
        this.onTouchedCallback = (/**
         * @return {?}
         */
        function () { });
        this.onChangeCallback = (/**
         * @return {?}
         */
        function () { });
        this.autoMaxHeight = 0;
        this.delayRefreshTime = 200;
        this.focusChange = new EventEmitter();
        this.codemirrorClassName = true;
    }
    Object.defineProperty(CodeMirrorComponent.prototype, "codeMirrorGlobal", {
        get: /**
         * @return {?}
         */
        function () {
            if (this._codeMirror) {
                return this._codeMirror;
            }
            this._codeMirror = typeof CodeMirror !== 'undefined' ? CodeMirror : require('codemirror');
            return this._codeMirror;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this._differ && this.options) {
            this._differ = this._differs.find(this.options).create();
        }
        this.initializeCodemirror();
        timer(this.delayRefreshTime).subscribe((/**
         * @return {?}
         */
        function () {
            if (_this.autoMaxHeight) {
                _this.initializeAutoMaxHeight();
            }
            _this.editor.refresh();
        }));
    };
    /**
     * @param {?} simpleChanges
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnChanges = /**
     * @param {?} simpleChanges
     * @return {?}
     */
    function (simpleChanges) {
        var _this = this;
        /** @type {?} */
        var optionsChange = simpleChanges.options;
        if (optionsChange && !optionsChange.firstChange) {
            /** @type {?} */
            var changes = this._differ.diff(this.options);
            if (changes) {
                changes.forEachChangedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
                changes.forEachAddedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
                changes.forEachRemovedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
            }
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    CodeMirrorComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onChangeCallback = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    CodeMirrorComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this.onTouchedCallback = fn;
    };
    /**
     * @param {?} value
     * @return {?}
     */
    CodeMirrorComponent.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (this.editor && this.value !== value) {
            this.editor.setValue(value);
        }
        this.value = value;
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.initializeCodemirror = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.ngZone.runOutsideAngular((/**
         * @return {?}
         */
        function () {
            if (!_this.options) {
                throw new Error("options is required");
            }
            _this.editor = _this.codeMirrorGlobal.fromTextArea(_this.textAreaRef.nativeElement, _this.options);
            _this.editor.on("focus", (/**
             * @return {?}
             */
            function () {
                return _this.ngZone.run((/**
                 * @return {?}
                 */
                function () { return _this.focusChange.emit(true); }));
            }));
            _this.editor.on("blur", (/**
             * @return {?}
             */
            function () {
                _this.ngZone.run((/**
                 * @return {?}
                 */
                function () { return _this.focusChange.emit(false); }));
                _this.editor.setCursor(0, null, { scroll: false });
            }));
            _this.editor.on("change", (/**
             * @param {?} cm
             * @param {?} change
             * @return {?}
             */
            function (cm, change) {
                if (change.origin !== "setValue") {
                    _this.value = cm.getValue();
                    _this.onChangeCallback(_this.value);
                }
            }));
        }));
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.initializeAutoMaxHeight = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._codeWrapElement = this.elementRef.nativeElement.querySelector(".CodeMirror-code");
        this._codemirrorContentObserver = new MutationObserver((/**
         * @return {?}
         */
        function () {
            _this.applyEditorHeight();
        }));
        this._codemirrorContentObserver.observe(this._codeWrapElement, {
            childList: true,
        });
    };
    /**
     * @private
     * @return {?}
     */
    CodeMirrorComponent.prototype.applyEditorHeight = /**
     * @private
     * @return {?}
     */
    function () {
        if (this._codeWrapElement.offsetHeight >= this.autoMaxHeight) {
            /** @type {?} */
            var wrapHeight = this.elementRef.nativeElement.querySelector(".CodeMirror");
            if (wrapHeight.offsetHeight > this.autoMaxHeight) {
                this.editor.setSize("100%", this.autoMaxHeight + "px");
            }
        }
        else {
            this.editor.setSize("100%", "auto");
        }
    };
    /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    CodeMirrorComponent.prototype.setOptionIfChanged = /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    function (optionName, newValue) {
        if (!this.editor) {
            return;
        }
        // cast to any to handle strictly typed option names
        // could possibly import settings strings available in the future
        this.editor.setOption((/** @type {?} */ (optionName)), newValue);
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this._codemirrorContentObserver) {
            this._codemirrorContentObserver.disconnect();
        }
    };
    CodeMirrorComponent.decorators = [
        { type: Component, args: [{
                    selector: "ng-codemirror, [ngCodeMirror]",
                    template: "<textarea #textAreaRef style=\"display:none;\"></textarea>",
                    preserveWhitespaces: false,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef((/**
                             * @return {?}
                             */
                            function () { return CodeMirrorComponent; })),
                            multi: true
                        }]
                }] }
    ];
    /** @nocollapse */
    CodeMirrorComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgZone },
        { type: Renderer2 },
        { type: KeyValueDiffers }
    ]; };
    CodeMirrorComponent.propDecorators = {
        autoMaxHeight: [{ type: Input }],
        options: [{ type: Input }],
        delayRefreshTime: [{ type: Input }],
        focusChange: [{ type: Output }],
        textAreaRef: [{ type: ViewChild, args: ["textAreaRef", { read: ElementRef, static: true },] }],
        codemirrorClassName: [{ type: HostBinding, args: ["class.ng-codemirror",] }]
    };
    return CodeMirrorComponent;
}());
if (false) {
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._codeWrapElement;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._codemirrorContentObserver;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.value;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.onTouchedCallback;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.onChangeCallback;
    /** @type {?} */
    CodeMirrorComponent.prototype.autoMaxHeight;
    /** @type {?} */
    CodeMirrorComponent.prototype.options;
    /** @type {?} */
    CodeMirrorComponent.prototype.delayRefreshTime;
    /** @type {?} */
    CodeMirrorComponent.prototype.focusChange;
    /** @type {?} */
    CodeMirrorComponent.prototype.textAreaRef;
    /** @type {?} */
    CodeMirrorComponent.prototype.codemirrorClassName;
    /** @type {?} */
    CodeMirrorComponent.prototype.editor;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differ;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._codeMirror;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.elementRef;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.ngZone;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.renderer;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differs;
}

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CodemirrorModule = /** @class */ (function () {
    function CodemirrorModule() {
    }
    CodemirrorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [CodeMirrorComponent],
                    entryComponents: [CodeMirrorComponent],
                    exports: [CodeMirrorComponent]
                },] }
    ];
    return CodemirrorModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ng-codemirror.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CodeMirrorComponent, CodemirrorModule };
//# sourceMappingURL=ng-codemirror.js.map
